
// 【统一的调用方式】：TrackerJSBridge.invoke(funcName, param, callback);与微信的调用方式对齐
// 接口都是异步的，通过回调函数返回执行结果
// @param funcName 要调的函数名，必填
// @param param object类型的参数，不需要参数时传入null，必填
// @param callback 回调函数， 不需要回调时传入null或不传，选填


// 【回调函数】function(res)中，res的err_msg字段用来判断是否调用成功，成功时err_msg=="ok"
	// res.err_msg的值
	// system:function_not_exist
	// ok
	// [fail message]

// 【返回值】只有一个时，通过res的ret字段获取，可以是各种类型
// 多个返回值时，扩展其他字段，使用时根据具体业务再定

/*	使用示例
	TrackerJSBridge.invoke("shareTimeline", {
		appid: "wx06e3073fd4d17e3b", 
		img_url:"http://s10.sinaimg.cn/orignal/001nH0jNgy6OEyrVMBb69", 
		img_width: "240", 
		img_height: "240", 
		link: "http://www.sina.com", 
		desc: "这是描述",
		title: "这是标题"
	}, function(res) { 
		log("err_msg: " + res.err_msg + " ret: " + res.ret);
	});
*/

// 日志开关
var gLogType = "console"; // alert console none
function _log(msg) {
	if ("none" == gLogType) {
		return;
	} else if ("alert" == gLogType) { // 弹框
		alert("[alert log] " + msg);
	} else if ("console" == gLogType) { // logcat
		try {
			console.log("[console log] " + msg);
		} catch(err) {
			// ignore
		}
	} else {
		// do nothing
	}
}

// session id，在网页和Android端之间通信时，作为网页的唯一标识，Android端onPageFinished的时候生成
var gSessionId = "";
// callbackId计数器，用来标识回调给哪个回调函数
var gCallbackId = 0;
// callbackId和对应的回调函数的映射
var gCallbackMap = new Map();
// 事件名和对应的回调函数的映射
var gEventMap = new Map();
// 用来调用客户端api的对象
var TrackerJSBridge = new TrackerJSBridgeImpl();

function setLogType(logType) {
	gLogType = logType;
}

function setSessionId(sessionId) {
	gSessionId = sessionId;
}

function TrackerJSBridgeImpl() {

	/**
	 * js调用客户端api的统一入口
	 * funcName 要调用的客户端函数名
	 * paramObj 调用参数，object类型，不需要参数时传入null或不传，如果有callback参数则此参数至少要传一个null
	 * callback 回调函数，不需要回调时传入null或不传
	 */
	this.invoke = function(funcName, paramObj, callback) {
		try {
			// _log("invoke: " + funcName);
		
			// 函数名合法性检查
			if (!funcName || typeof funcName != "string" || funcName == "") {
				throw new Error("'funcName' must be a non empty string: " + funcName);
			}
		
			// 参数检查并转换为string
			var paramStr = null;
			if (paramObj) {
				if (typeof paramStr == "object") {
					paramStr = JSON.stringify(paramObj);
					// _log("JSON.stringify(paramObj): " + paramStr);
				} else {
					throw new Error("unsupported param type for 'paramObj': " + paramObj);
				}
			}
			
			// 回调函数类型检查
			if (!callback) {
				// 没定义当成不需要回调
				// throw new Error("callback is unDefined or null: " + callback);
				_log("Warning: callback is undefined or null: " + callback);
			} else if (typeof callback != "function") {
				// 有定义又不是函数类型，类型错误
				throw new Error("'callback' must be a function: " + callback);
			}
			
			// 生成callbackId并调用Android函数
			var localCallbackId = ++gCallbackId;
			gCallbackMap.put(localCallbackId, callback);

			// 直接调java对象的接口会有漏洞，改成调用js自带的confirm()方法，
			// 并在参数中传入特殊标记，android端捕获onJsConfirm()事件判断是否有特殊标记，然后转成调用相应的函数
			//window.TcsJSApi.invoke(localCallbackId, funcName, paramStr);
			confirm(genInvokeInfoStr(localCallbackId, funcName, paramStr));
		} catch (err) {
			_log("invoke err: " + err.message);
			throw err;
		}
		
	}

	/**
	 * js向客户端注册事件监听的统一入口
	 * eventName 事件名
	 * callback 回调函数
	 */
	this.on = function(eventName, callback) {
		try {
			// 函数名合法性检查
			if (!eventName || typeof eventName != "string" || eventName == "") {
				throw new Error("'eventName' must be a non empty string: " + eventName);
			}
		
			// 回调函数类型检查
			if (!callback) {
				throw new Error("'callback' is unDefined or null: " + callback);
			} else if (typeof callback != "function") {
				// 有定义又不是函数类型，类型错误
				throw new Error("'callback' must be a function: " + callback);
			}
			
			var listenerList = gEventMap.get(eventName);
			var isNew = false;
			if (listenerList == null) {
				listenerList = new Array();
				gEventMap.put(eventName, listenerList);
				isNew = true;
			}
			listenerList.push(callback);
			if (isNew) {
				// 向客户端注册事件
				
				// 直接调java对象的接口会有漏洞，改成调用js自带的confirm()方法，
				// 并在参数中传入特殊标记，android端捕获onJsConfirm()事件判断是否有特殊标记，然后转成调用相应的函数
				// window.TcsJSApi.on(eventName);
				confirm(genOnInfoStr(eventName));
			}
		} catch (err) {
			_log("on err: " + err.message);
			throw err;
		}
		
	}
	
	/**
	 * 将调用接口的操作拼成字符串，java端将解析到是一个调用接口的操作
	 */
	function genInvokeInfoStr(callbackId, funcName, paramStr) {
		var invokeInfoObj = {
			"sessionId" : gSessionId, 
			"callbackId" : callbackId, 
			"funcName" : funcName, 
			"paramStr" : paramStr
		};
		var invokeInfoStr = "#js_invoke#" +  JSON.stringify(invokeInfoObj);
		return invokeInfoStr;
	}
	
	/**
	 * 将注册事件的操作拼成字符串，java端将解析到是一个注册事件的操作
	 */
	function genOnInfoStr(eventName) {
		var onInfoObj = {
			"sessionId" : gSessionId, 
			"eventName" : eventName
		};
		var onInfoStr = "#js_on#" +  JSON.stringify(onInfoObj);
		return onInfoStr;
	}
}

/**
 * 处理客户端事件：js库初始化完毕
 * paramStr 客户端给回的参数，需要转换成JSON对象
 */
function onInit(paramStr) {
	try {
		// 设日志开关、sessionId
		if (paramStr && typeof paramStr == "string" && paramStr != "") {
			var res = JSON.parse(paramStr);
			if (res.logType) {
				setLogType(res.logType);
			}
			if (res.sessionId) {
				setSessionId(res.sessionId);
			}
		}
	} catch (err) {
		_log("onInit1 err: " + err.message);
	}
	
	try {
		// 避免java层多次发起init事件，造成网页端多次收到
		if (window.TrackerJSBridge._hasInit) {
			_log('hasInit, no need to init again');
			return;
		}
		
		window.TrackerJSBridge._hasInit = true;
		
		// 发出ready事件
		// _log("dispatchEvent: TrackerJSBridgeReady");
        var readyEvent = document.createEvent('Events');
        readyEvent.initEvent('TrackerJSBridgeReady');
        document.dispatchEvent(readyEvent);
	} catch (err) {
		_log("onInit2 err: " + err.message);
	}
}

/**
 * 客户端回调给js的统一入口
 * paramStr 客户端给回的参数，需要转换成JSON对象
 */
function onCallback(paramStr) {
	try {
		var res = JSON.parse(paramStr);
		// sessionId校验
		if (res.sessionId && res.sessionId != gSessionId) {
			return;
		}
		
		// js调客户端时传入的回调函数id
		var callbackId = res.callbackId;
		// _log("onCallback: " + callbackId + " " + paramStr);
		var callback = gCallbackMap.get(callbackId);
		if (callback != null) {
			gCallbackMap.remove(callbackId);
			callback(res);
		}
	} catch (err) {
		_log("onCallback err: " + err.message);
	}
}

/**
 * 客户端发布事件接收处理
 * paramStr 客户端给回的参数，需要转换成JSON对象
 */
function onReceive(paramStr) {
	try {
		var res = JSON.parse(paramStr);
		// sessionId校验
		if (res.sessionId && res.sessionId != gSessionId) {
			return;
		}
		
		// js向客户端注册事件时传入的事件名
		var eventName = res.eventName;
		// _log("onReceive: " + paramStr + " " + paramStr);
		var listenerList = gEventMap.get(eventName);
		if (listenerList != null) {
			var len = listenerList.length;
			for (var i = 0; i < len; i++) {
				listenerList[i](res);
			}
		}
	} catch (err) {
		_log("onReceive err: " + err.message);
	}
}

/**
 * 客户端发到js的事件统一入口
 */
function handleMessageFromTracker(messageName, paramStr) {
	if (messageName == "sys:init") {
		onInit(paramStr);
	} else if (messageName == "onCallback") {
		onCallback(paramStr);
	} else if (messageName == "event:broadcast") {
		onReceive(paramStr);
	}
}

/**
 * 实现MAP的功能
 *
 * 例子：
 * var map = new Map();
 *
 * map.put("key", "value");
 * var val = map.get("key")
 * ……
 *
 */
function Map() {
	this.map    =  new Object();
	this.length = 0;
    
	this.size = function() {
		return this.length;
	}  
    
	this.put = function(key, value){
		if( !this.map['_' + key]) {
			++this.length;
		}  
       
		this.map['_' + key] = value;
	}  
    
	this.remove = function(key){
		if(this.map['_' + key]) {
			--this.length;  
			return delete this.map['_' + key]; 
		} else {  
			return false;  
		}  
	}  
    
	this.containsKey = function(key){
		return this.map['_' + key] ? true : false;
	}
    
	this.get = function(key){     
		return this.map['_' + key] ? this.map['_' + key] : null;  
	}  
  
  
	this.toString = function() {
		var str = '';
    
		for(var each in this.map) {
			str += '\n' + each + '-' + this.map[each];
		}
    
		return str;
	}
}
